@@ -30,13 +30,15 @@
 #define FIO_HAVE_SYSLET
 
 #define nop		__asm__ __volatile__("rep;nop": : :"memory")
-#define read_barrier()	asm volatile("lfence":::"memory")
+#define read_barrier()	__asm__ __volatile__("lfence":::"memory")
+#define write_barrier()	__asm__ __volatile__("sfence":::"memory")
 
-static inline unsigned long fio_ffz(unsigned long bitmask)
+static inline unsigned int arch_ffz(unsigned int bitmask)
 {
-	__asm__("bsfq %1,%0" :"=r" (bitmask) :"r" (~bitmask));
+	__asm__("bsfl %1,%0" :"=r" (bitmask) :"r" (~bitmask));
 	return bitmask;
 }
-
+#define ARCH_HAVE_FFZ
+#define ARCH_HAVE_SSE
 
 #endif
