@@ -1,6 +1,8 @@
 #ifndef ARCH_H
 #define ARCH_H
 
+#define BITS_PER_LONG	__WORDSIZE
+
 enum {
 	arch_x86_64,
 	arch_i386,
@@ -31,6 +33,8 @@ enum {
 #include "arch-sparc64.h"
 #elif defined(__arm__)
 #include "arch-arm.h"
+#elif defined(__mips__) || defined(__mips64__)
+#include "arch-mips.h"
 #else
 #error "Unsupported arch"
 #endif
