@@ -223,7 +223,7 @@ static int __check_min_rate(struct thread_data *td, struct timeval *now,
 				if (rate < ratemin ||
 				    bytes < td->rate_bytes[ddir]) {
 					log_err("%s: min rate %u not met, got"
-						" %luKiB/sec\n", td->o.name,
+						" %luKB/sec\n", td->o.name,
 							ratemin, rate);
 					return 1;
 				}
@@ -536,7 +536,8 @@ sync_done:
 		 */
 		full = queue_full(td) || ret == FIO_Q_BUSY;
 		if (full || !td->o.iodepth_batch_complete) {
-			min_events = td->o.iodepth_batch_complete;
+			min_events = min(td->o.iodepth_batch_complete,
+					 td->cur_depth);
 			if (full && !min_events)
 				min_events = 1;
 
@@ -581,7 +582,8 @@ static void do_io(struct thread_data *td)
 	else
 		td_set_runstate(td, TD_RUNNING);
 
-	while ((td->this_io_bytes[0] + td->this_io_bytes[1]) < td->o.size) {
+	while ( (td->o.read_iolog_file && !flist_empty(&td->io_log_list)) ||
+	        ((td->this_io_bytes[0] + td->this_io_bytes[1]) < td->o.size) ) {
 		struct timeval comp_time;
 		unsigned long bytes_done[2] = { 0, 0 };
 		int min_evts = 0;
@@ -606,7 +608,8 @@ static void do_io(struct thread_data *td)
 		 * Add verification end_io handler, if asked to verify
 		 * a previously written file.
 		 */
-		if (td->o.verify != VERIFY_NONE && io_u->ddir == DDIR_READ) {
+		if (td->o.verify != VERIFY_NONE && io_u->ddir == DDIR_READ &&
+		    !td_rw(td)) {
 			if (td->o.verify_async)
 				io_u->end_io = verify_io_u_async;
 			else
@@ -686,7 +689,8 @@ sync_done:
 		 */
 		full = queue_full(td) || ret == FIO_Q_BUSY;
 		if (full || !td->o.iodepth_batch_complete) {
-			min_evts = td->o.iodepth_batch_complete;
+			min_evts = min(td->o.iodepth_batch_complete,
+					td->cur_depth);
 			if (full && !min_evts)
 				min_evts = 1;
 
@@ -1105,7 +1109,10 @@ static void *thread_main(void *data)
 	clear_state = 0;
 	while (keep_running(td)) {
 		fio_gettime(&td->start, NULL);
-		memcpy(&td->ts.stat_sample_time, &td->start, sizeof(td->start));
+		memcpy(&td->ts.stat_sample_time[0], &td->start,
+				sizeof(td->start));
+		memcpy(&td->ts.stat_sample_time[1], &td->start,
+				sizeof(td->start));
 		memcpy(&td->tv_cache, &td->start, sizeof(td->start));
 
 		if (td->o.ratemin[0] || td->o.ratemin[1])
@@ -1190,6 +1197,10 @@ err:
 	if (td->error)
 		printf("fio: pid=%d, err=%d/%s\n", (int) td->pid, td->error,
 							td->verror);
+
+	if (td->o.verify_async)
+		verify_async_exit(td);
+
 	close_and_free_files(td);
 	close_ioengine(td);
 	cleanup_io_u(td);
@@ -1200,9 +1211,6 @@ err:
 		td_verror(td, ret, "fio_cpuset_exit");
 	}
 
-	if (td->o.verify_async)
-		verify_async_exit(td);
-
 	/*
 	 * do this very late, it will log file closing as well
 	 */
@@ -1615,12 +1623,6 @@ int main(int argc, char *argv[])
 	if (!getenv("LC_NUMERIC"))
 		setlocale(LC_NUMERIC, "en_US");
 
-	if (parse_options(argc, argv))
-		return 1;
-
-	if (!thread_number)
-		return 0;
-
 	ps = sysconf(_SC_PAGESIZE);
 	if (ps < 0) {
 		log_err("Failed to get page size\n");
@@ -1630,6 +1632,14 @@ int main(int argc, char *argv[])
 	page_size = ps;
 	page_mask = ps - 1;
 
+	fio_keywords_init();
+
+	if (parse_options(argc, argv))
+		return 1;
+
+	if (!thread_number)
+		return 0;
+
 	if (write_bw_log) {
 		setup_log(&agg_io_log[DDIR_READ]);
 		setup_log(&agg_io_log[DDIR_WRITE]);
